<?php

namespace Teknisa\FIN\Modules\ContaPagar\Controllers;

use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Teknisa\FIN\Modules\ContaPagar\Services\ParcelamentoTituloService;

/**
 * Class ParcelamentoTituloController
 * @package Teknisa\FIN\Modules\ContaPagar\Controllers
 */
class ParcelamentoTituloController extends TeknisaCrud{

    protected $dataSourceName = 'boletpagar_motivoparc_contcorr';
    protected $parcelamentoTituloService;

    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param ParcelamentoTituloService $parcelamentoTituloService
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, ParcelamentoTituloService $parcelamentoTituloService)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->parcelamentoTituloService = $parcelamentoTituloService;
    }

    /**
     * Finaliza Parcelamento Títulos
     * @param Data $dataset
     * @param Response $response
     */
    public function finalizaParcelamentoTitulo(Data $dataset, Response $response)
    {
        try {
            set_time_limit(0);
            
            // Dados do Registro
            $dataset  = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $nrorg       = null;
            $cdfilial    = null;
            $nrlanbolpag = null;
            $parcelas = array();

            // Percorre Parcelas
            foreach($dataset as $data){
                // Verifica Parcela Inicial
                if(($data['NRPARCBOLPAG'] == '00') || ($data['NRPARCBOLPAG'] == '01')) {
                    $nrorg       = $data['NRORG'];
                    $cdfilial    = $data['CDFILIAL'];
                    $nrlanbolpag = $data['NRLANBOLPAG'];
                }

                // Monta Array Parcelas
                $parcelas[] = array(
                    'NRORG'            => $data['NRORG']       ,
                    'CDFILIAL'         => $data['CDFILIAL']    ,
                    'NRLANBOLPAG'      => $data['NRLANBOLPAG'] ,
                    'NRORIGBOLPAG'     => $nrlanbolpag,
                    'NRPARCBOLPAG'     => $data['NRPARCBOLPAG'],
                    'NRPARCBOLPAGVISU' => $data['NRPARCBOLPAGVISU'],
                    'DTVENBOLPAG'      => $data['DTVENBOLPAG'] ,
                    'VRFINALBPAG'      => floatval($data['VRFINALBPAG']) ,
                    'CDMOTIPARC'       => isset($data['CDMOTIPARC'])?$data['CDMOTIPARC']:null
                );
            }
            
            // Gera Parcelamento Título a Pagar/Pagos
            $this->parcelamentoTituloService->geraParcelamentoTitulo(
                $nrorg, $cdfilial, $nrlanbolpag, $parcelas
            );

            // Registra Dados
            $response->addDataSet(new DataSet('finalizaParcelamentoTitulo', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Deleta Parcelamento Título
     * @param Data $dataset
     * @param Response $response
     */
    public function deletaParcelamentoTitulo(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $dataset  = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $parcelas = array();

            // Percorre Parcelas
            foreach($dataset as $data){
                // Monta Array Parcelas
                $parcelas[] = array(
                    'NRORG'        => $data['NRORG']       ,
                    'CDFILIAL'     => $data['CDFILIAL']    ,
                    'NRLANBOLPAG'  => $data['NRLANBOLPAG'] ,
                    'NRORIGBOLPAG' => $data['NRORIGBOLPAG'],
                    'NRPARCBOLPAG' => $data['NRPARCBOLPAG'],
                    'DTVENBOLPAG'  => $data['DTVENBOLPAG'] ,
                    'VRFINALBPAG'  => floatval($data['VRFINALBPAG']) ,
                    'CDMOTIPARC'   => $data['CDMOTIPARC']
                );
            }

            // Valida Parcela Atual
            $boletoatual = $this->parcelamentoTituloService->validaParcelamentoTitulo(
                $parcelas[count($parcelas)-1]['NRORG'],
                $parcelas[count($parcelas)-1]['CDFILIAL'],
                $parcelas[count($parcelas)-1]['NRLANBOLPAG']
            );

            // Valida Parcela Anterior
            $boletoant   = $this->parcelamentoTituloService->validaParcelamentoTitulo(
                $parcelas[count($parcelas)-2]['NRORG'],
                $parcelas[count($parcelas)-2]['CDFILIAL'],
                $parcelas[count($parcelas)-2]['NRLANBOLPAG']);

            // Deleta Parcelamento do Título
            $this->parcelamentoTituloService->deletaParcelamentoTitulo($boletoant,$boletoatual,count($parcelas));

            // Registra Dados
            $response->addDataSet(new DataSet('ParcelamentoBoletoService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }


}